<?php
/**
 * Autogenerated by Thrift
 *
 * DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
 */
include_once $GLOBALS['THRIFT_ROOT'].'/Thrift.php';

include_once $GLOBALS['THRIFT_ROOT'].'/packages/reflection_limited/reflection_limited_types.php';

$GLOBALS['E_fb_status'] = array(
  'DEAD' => 0,
  'STARTING' => 1,
  'ALIVE' => 2,
  'STOPPING' => 3,
  'STOPPED' => 4,
  'WARNING' => 5,
);

final class fb_status {
  const DEAD = 0;
  const STARTING = 1;
  const ALIVE = 2;
  const STOPPING = 3;
  const STOPPED = 4;
  const WARNING = 5;
  public static $__names = array(
    0 => 'DEAD',
    1 => 'STARTING',
    2 => 'ALIVE',
    3 => 'STOPPING',
    4 => 'STOPPED',
    5 => 'WARNING',
  );
}

?>
